package test;

import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.nio.file.Paths;
import java.util.Arrays;

public class TestJDKClassUnloadingPerformance {

    private static final int NUMBER_OF_CLASSES_IN_JAR = 1_000;

    /**
     * For a specified iterations count, loads a specified number of classes.
     *
     * Classes are loaded with different class loaders.
     *
     * @param args
     *            At position 0: path to the classes.jar, which contains classes to load.
     *            At position 1: number of iterations.
     *            At position 2: total number of loaded classes before unloading begins. Minimum is 1000.
     */
    public static void main(String[] args) throws Exception {
        String classesJarPath = args[0];
        URL[] urls = {
                Paths.get(classesJarPath).toUri().toURL()
        };

        int iterations = Integer.valueOf(args[1]);
        int numberOfClasses = Integer.valueOf(args[2]);
        if (numberOfClasses < 1000) {
            throw new IllegalArgumentException("Expected at least 1000 classes to load, arguments are: " + Arrays.toString(args));
        }

        int numberOfClassLoaders = numberOfClasses / NUMBER_OF_CLASSES_IN_JAR;
        long start = System.currentTimeMillis();

        for (int iteration = 0; iteration < iterations; ++iteration) {
            System.out.print("Iteration " + iteration + "/" + iterations + "... ");
            URLClassLoader[] classLoaders = new URLClassLoader[numberOfClassLoaders];
            for (int i = 0; i < numberOfClassLoaders; ++i) {
                classLoaders[i] = new URLClassLoader(urls, null);
            }
            for (URLClassLoader classLoader : classLoaders) {
                loadTestClasses(classLoader);
            }
            for (int i = 0; i < numberOfClassLoaders; ++i) {
                classLoaders[i].close();
                classLoaders[i] = null;
                System.gc();
                System.runFinalization();
            }
            classLoaders = null;
            System.gc();
            System.runFinalization();
            System.out.println("done.");
        }

        long end = System.currentTimeMillis();
        long elapsed = end - start;
        System.out.println("Total time: " + elapsed + " ms");
    }

    private static void loadTestClasses(URLClassLoader classLoader)
            throws ClassNotFoundException, InstantiationException, IllegalAccessException {
        for (int j = 0; j < NUMBER_OF_CLASSES_IN_JAR; ++j) {
            String className = "sample.C" + j;
            Class<?> clazz = classLoader.loadClass(className);
            clazz.newInstance();
        }
    }
}
